/*$Id: expression.h,v 1.43 2001/06/15 21:53:20 al Exp $ -*- C++ -*-
 */
#include "base.h"
/*--------------------------------------------------------------------------*/
class Symbol_Table;
class Macro_Body;
class Instance;
#define NOTHING 0
/*--------------------------------------------------------------------------*/
class Token
  :public Base
{
public:
  enum Type {UNKNOWN, SYMBOL, BINOP, STOP, PARLIST, UNARY, CONSTANT};
private:
  std::string _name;
  Type	      _type;
  const Base* _data;
  std::string _args;
  void parse(CS&) {unreachable();}
  void dump(std::ostream&)const;
public:
  explicit Token(const std::string& n, Type t, const Base* d,
		 const std::string& a)
    : _name(n), _type(t), _data(d), _args(a) {}
  explicit Token(const Token& p)
    : Base(), _name(p._name), _type(p._type), _data(p._data), _args(p._args) {}
  const std::string& name()const {return _name;}
  Type		     type()const {return _type;}
  const Base*	     data()const {return _data;}
  const std::string& args()const {return _args;}
  const std::string  full_name()const {return name() + args();}
  Token*	     op(const Token* t1, const Token* t2)const;
  Token*	     op(const Token* t1)const;
};
/*--------------------------------------------------------------------------*/
class Expression
  :public List_Base<Token>
{
private:
  Symbol_Table* _sym;
  std::string _scope;
private: // override virtual
  void parse(CS&);
  void dump(std::ostream&)const;
private: // expression-in.cc
  void arglisttail(CS& file);
  void arglist(CS& file);
  void leaf(CS& file);
  void factor(CS& file);
  void term(CS& file);
  void addexp(CS& file);
  void logical(CS& file);
  void expression(CS& file);
public:
  explicit Expression(const Macro_Body* m);
private: // expression-reduce.cc
  void reduce_copy(const Expression&, const Instance&);
public:
  explicit Expression(const Expression&, const Instance&);
public: // other
  bool as_bool()const {return (!is_empty() && back()->data());}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
